mod nav_bar;
mod song_edit_window;


use egui::{Color32, Label, Sense};
use egui_extras::{Column, TableBuilder};

use crate::{config::{Config, ConfigWrapper}, downloader::Downloader, manifest::{song::SongType, Manifest}};

use self::song_edit_window::GuiSongEditor;


#[derive(Debug)]
pub struct Gui {
    manifest: Manifest,
    song_editor: GuiSongEditor,
    filter: String,
    downloader: Downloader,
    cfg: ConfigWrapper
}

impl Gui {
    fn new(_: &eframe::CreationContext<'_>, manifest: Manifest, downloader: Downloader, cfg: ConfigWrapper) -> Self {
        Self {
            manifest,
            song_editor: GuiSongEditor {
                is_new_open: false,
                is_open: false,
                song: Default::default(),
                ed_url: String::new(),
                ed_name: String::new(),
                ed_playlist: Some(String::new()),
                ed_type: SongType::Youtube
            },
            filter: String::new(),
            downloader,
            cfg,
        }
    }

    pub fn start(manifest: Manifest, downloader: Downloader, cfg: ConfigWrapper) -> anyhow::Result<()> {
        let native_options = eframe::NativeOptions {
            viewport: egui::ViewportBuilder::default()
                .with_inner_size([400.0, 300.0])
                .with_min_inner_size([300.0, 220.0]),
                // .with_icon(
                //     // NOTE: Adding an icon is optional
                //     eframe::icon_data::from_png_bytes(&include_bytes!("../assets/icon-256.png")[..])
                //         .expect("Failed to load icon"),
                // ),
            ..Default::default()
        };

        if let Err(e) = eframe::run_native(
            "eframe template",
            native_options,
            Box::new(|cc| Box::new(Gui::new(cc, manifest, downloader, cfg))),
        ) {
            log::error!("Failed to create window: {e}");
        };

        Ok(())
    }
}

impl eframe::App for Gui {
    fn update(&mut self, ctx: &egui::Context, frame: &mut eframe::Frame) {
        self.draw_nav(ctx, frame);
        self.draw_song_edit_window(ctx, frame);
        self.draw_new_song_window(ctx, frame);

        egui::CentralPanel::default().show(ctx, |ui| {
            // The central panel the region left after adding TopPanel's and SidePanel's
            //ui.heading(format!("Songs ({})", self.manifest.get_song_count()));
            
            let fltr_by;
            let filter_clean;
            if self.filter.starts_with("playlist:") {
                fltr_by = "playlist";
                filter_clean = self.filter.strip_prefix("playlist:").unwrap_or("").to_string().to_lowercase();
            } else if self.filter.starts_with("source:") {
                fltr_by = "source";
                filter_clean = self.filter.strip_prefix("source:").unwrap_or("").to_string().to_lowercase();
            } else if self.filter.starts_with("url:") {
                fltr_by = "url";
                filter_clean = self.filter.strip_prefix("url:").unwrap_or("").to_string();
            } else {
                fltr_by = "";
                filter_clean = self.filter.clone();
            }

            ui.vertical(|ui| {
                ui.horizontal(|ui| {
                    ui.colored_label(Color32::BLUE, "Filter: ");
                    ui.text_edit_singleline(&mut self.filter);
                });  
            });
            
            ui.vertical(|ui| {
                let available_height = ui.available_height();
                let table = TableBuilder::new(ui)
                    .striped(true)
                    .cell_layout(egui::Layout::left_to_right(egui::Align::Center))
                    .resizable(true)
                    .column(Column::auto())
                    .column(Column::auto())
                    //.column(
                    //    Column::remainder()
                    //        .at_least(40.0)
                    //        .clip(true)
                    //        .resizable(true),
                    //)
                    .column(Column::auto())
                    .column(Column::remainder())
                    //.column(Column::remainder())
                    .min_scrolled_height(0.0)
                    .max_scroll_height(available_height)
                    .sense(egui::Sense::click());
                
                let playlists = self.manifest.get_playlists().clone();

                let songs = {
                    let mut songs = Vec::new();
                    for (pname, p) in playlists {
                        for (sname, s) in p {
                            songs.push((pname.clone(), sname, s))
                        }
                    }
                    songs
                };

                table.header(20.0, |mut header| {
                    header.col(|_|{});
                    header.col(|ui| {
                        ui.strong("Playlist");
                    }); 
                    header.col(|ui| { 
                        ui.strong("Source");
                    });
                    header.col(|ui| {
                        ui.strong("Name");
                    });
                }).body(|mut body| {
                    for (pname, sname, s) in songs {    
                        if fltr_by == "playlist" && !filter_clean.is_empty() {
                            if !pname.to_lowercase().contains(&filter_clean) {
                                continue;
                            }
                        } else if fltr_by == "type" && !filter_clean.is_empty(){
                            if !s.get_type().to_string().to_lowercase().contains(&filter_clean) {
                                continue;
                            }
                        } else if fltr_by == "url" && !filter_clean.is_empty(){
                            if !s.get_url_str().contains(&filter_clean) {
                                continue;
                            }
                        } else if !filter_clean.is_empty() {
                            if !sname.to_lowercase().contains(&filter_clean) {
                                continue;
                            }
                        }

                        body.row(18.0, |mut row| {
                            row.col(|ui| {
                                if ui.add(Label::new("[edit]").sense(Sense::click())).clicked() {
                                    self.song_editor.song = (
                                        pname.clone(),
                                        sname.clone(),
                                    );
                                    log::debug!("Label pressed");
                                    self.song_editor.is_open = true;
                                    self.song_editor.ed_name = sname.clone();
                                    self.song_editor.ed_url = s.get_url_str().clone();
                                }
                
                            });

                            row.col(|ui| {
                                ui.label(pname.clone());
                            });
                            row.col(|ui| {
                                ui.label(s.get_type().to_string());
                            });
                            row.col(|ui| {
                                ui.hyperlink_to(sname.clone(), s.get_url_str());
                            });
                        })
                    }
                });
            });
            ui.separator();


            ui.with_layout(egui::Layout::bottom_up(egui::Align::LEFT), |ui| {
                egui::warn_if_debug_build(ui);
            });
        });
    }
}
